package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 15.3.2015

// This app demonstrates the LayoutTransition feature of Android, i.e. the animated adding and removal of layout items.

import android.animation.LayoutTransition;
import android.animation.ObjectAnimator;
import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.Date;

public class LayoutTransitionActivity extends Activity {

    LinearLayout leftColumn, rightColumn;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.layout_transition);

        // CheckBox to activate/deactivate layout animations with
        // Initially, animations are activated - see attribute android:animateLayoutChanges in layout_transition.xml
        CheckBox animationOnOffCheckbox = (CheckBox) findViewById(R.id.animationOnOffCheckbox);
        animationOnOffCheckbox.setOnCheckedChangeListener(new OnOffCheckboxListener());

        // The two vertical linear layouts to be animated
        leftColumn = (LinearLayout) findViewById(R.id.leftColumn);
        rightColumn = (LinearLayout) findViewById(R.id.rightColumn);

//        To selectively enable/disable specific type of animations, i.e. animations of appearing/disappearing/... layout items
//        leftColumn.getLayoutTransition().enableTransitionType(LayoutTransition.APPEARING);
//        leftColumn.getLayoutTransition().enableTransitionType(LayoutTransition.DISAPPEARING);
//        leftColumn.getLayoutTransition().enableTransitionType(LayoutTransition.CHANGE_APPEARING);
//        leftColumn.getLayoutTransition().enableTransitionType(LayoutTransition.CHANGE_DISAPPEARING);
//        leftColumn.getLayoutTransition().enableTransitionType(LayoutTransition.CHANGING);

        // To control the speed of the animation
        leftColumn.getLayoutTransition().setDuration(1000);
        rightColumn.getLayoutTransition().setDuration(1000);

//        To selectively control the speeds of different types of animations
//        leftColumn.getLayoutTransition().setDuration(LayoutTransition.APPEARING,1000);
//        leftColumn.getLayoutTransition().setDuration(LayoutTransition.DISAPPEARING,1000);
//        leftColumn.getLayoutTransition().setDuration(LayoutTransition.CHANGE_APPEARING,1000);
//        leftColumn.getLayoutTransition().setDuration(LayoutTransition.CHANGE_DISAPPEARING,1000);
//        leftColumn.getLayoutTransition().setDuration(LayoutTransition.CHANGING,1000);

//        To selectively control the start delays of different types of animations
//        leftColumn.getLayoutTransition().setStartDelay(LayoutTransition.APPEARING, 1000);
//        rightColumn.getLayoutTransition().setStartDelay(LayoutTransition.APPEARING, 1000);

    }

    // Method to move an item from the right to the left vertical layout

    public void moveViewLeft(View v) {

        switch (rightColumn.getChildCount()) {
            // origin and target position of the moving view depend on whether there are still two views or only one view left on the right-hand side
            case 1:
                TextView viewToMove = (TextView) rightColumn.getChildAt(0);
                rightColumn.removeView(viewToMove);
                leftColumn.addView(cloneTextView(viewToMove),1);  // add a clone of the disappearing view, not the disappearing view itself!
                                                                  // otherwise the two animations (disappearing on the right, appearing on the left) would not work.
                return;
            case 2:
                viewToMove = (TextView) rightColumn.getChildAt(1);
                rightColumn.removeView(viewToMove);
                leftColumn.addView(cloneTextView(viewToMove),2);
                return;
        }

   }

    // Method to move an item from the left to the right vertical layout (for comments see moveViewLeft())

    public void moveViewRight(View v) {

        switch (rightColumn.getChildCount()) {
            case 0:
                TextView viewToMove = (TextView) leftColumn.getChildAt(1);
                leftColumn.removeView(viewToMove);
                rightColumn.addView(cloneTextView(viewToMove),0);
                return;
            case 1:
                viewToMove = (TextView) leftColumn.getChildAt(2);
                leftColumn.removeView(viewToMove);
                rightColumn.addView(cloneTextView(viewToMove),1);
                return;
        }

    }

    // Method to add a new view on top of the left vertical layout

    public void addView(View v) {

        // Add a listener to become active when an animation (= a transition) starts or ends (only if animations are enabled and the listener is not already there).
        // This listener, which is defined below, just tells via LogCat when the respective transitions start and end.
        if (leftColumn.getLayoutTransition()!=null && leftColumn.getLayoutTransition().getTransitionListeners().size()<=1)
            leftColumn.getLayoutTransition().addTransitionListener(new MyTransitionListener());

        TextView newView = cloneTextView((TextView) leftColumn.getChildAt(0));
        newView.setText(R.string.zero);
        leftColumn.addView(newView,0);

    }

    // Method to add a new view on top of the left vertical layout
    // More comments see addView()

    public void removeView(View v) {

        if (leftColumn.getLayoutTransition()!=null && leftColumn.getLayoutTransition().getTransitionListeners().size()<=1)
            leftColumn.getLayoutTransition().addTransitionListener(new MyTransitionListener());

        // Add an animator that will move the disappearing view to the right
        if (leftColumn.getLayoutTransition()!=null) {
            leftColumn.getLayoutTransition().setAnimator(
                    LayoutTransition.DISAPPEARING,
                    ObjectAnimator.ofFloat(null,"X",500));
        }

        TextView viewToRemove = (TextView) leftColumn.getChildAt(0);
        if (!viewToRemove.getText().equals(getResources().getString(R.string.zero))) return;
        leftColumn.removeViewAt(0);

    }

    // Auxiliary method and classes

    // Method to clone a TextView

    private TextView cloneTextView(TextView tv) {
        TextView newView = new TextView(tv.getContext());
        newView.setText(tv.getText());
        newView.setLayoutParams(tv.getLayoutParams());
        newView.setTextColor(tv.getCurrentTextColor());
        newView.setTextSize(TypedValue.COMPLEX_UNIT_PX,tv.getTextSize());  // "TypedValue.COMPLEX_UNIT_PX": needed because getTextSize() returns text size in px, setTextSize() normally expects it in sp (= scaled pixels)
        return newView;
    }

    // Listener for the CheckBox: Activates/deactivates animations

    private class OnOffCheckboxListener implements CompoundButton.OnCheckedChangeListener {

        LayoutTransition leftColumnTransitionSaved, rightColumnTransitionSaved;  // to save the LayoutTransition objects when animations are disabled

        public void onCheckedChanged(CompoundButton button, boolean isChecked) {
            if (isChecked) {
                leftColumn.setLayoutTransition(leftColumnTransitionSaved);  // restore the formerly defined LayoutTransitions
                rightColumn.setLayoutTransition(rightColumnTransitionSaved);
            }
            else {
                leftColumnTransitionSaved = leftColumn.getLayoutTransition();  // save current LayoutTransitions ...
                rightColumnTransitionSaved = rightColumn.getLayoutTransition();
                leftColumn.setLayoutTransition(null);  // ... and disable animations
                rightColumn.setLayoutTransition(null);
            }
        }
    }

    // Listener that becomes active when a transition (= animation of a layout item) starts and ends

    private class MyTransitionListener implements LayoutTransition.TransitionListener {

        // auxiliary method taking integer-coded transition types and returning corresponding strings

        String transitionTypeToText(int transitionType) {
            switch (transitionType) {
                case LayoutTransition.APPEARING: return "APPEARING";
                case LayoutTransition.DISAPPEARING: return "DISAPPEARING";
                case LayoutTransition.CHANGE_APPEARING: return "CHANGE_APPEARING";
                case LayoutTransition.CHANGE_DISAPPEARING: return "CHANGE_DISAPPEARING";
                case LayoutTransition.CHANGING: return "CHANGING";
            }
            return "UNDEFINED";
        }

        // method called by the run-time system when a transition (= animation of a layout item) starts

        @Override
        public void startTransition(LayoutTransition transition, ViewGroup container, View view, int transitionType) {
            Log.v("MYLOG", transitionTypeToText(transitionType) + " animation starting at " + (new Date()).getTime()%100000 + "ms");
        }

        // method called by the run-time system when a transition (= animation of a layout item) ends

        @Override
        public void endTransition(LayoutTransition transition, ViewGroup container, View view, int transitionType) {
            Log.v("MYLOG", transitionTypeToText(transitionType) + " animation ending at " + (new Date()).getTime()%100000 + "ms");
        }

    }

}
