package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 15.11.2017

// This activity demonstrates how to use ObjectAnimators and an AnimatorSet to animate multiple properties of a view with constant speed.

import android.animation.AnimatorSet;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.animation.ObjectAnimator;
import android.widget.TextView;


public class MultiplePropertiesAnimationActivity extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.multiple_properties_animation);

    }

    public void startAnimation(View v) {

        int duration = 2000;  // duration of the animation in ms
        int screenWidth = getResources().getDisplayMetrics().widthPixels;
        int screenHeight = getResources().getDisplayMetrics().heightPixels;

        TextView animatedView = (TextView) findViewById(R.id.viewMultiAnim);
        float endX = screenWidth - animatedView.getWidth() - 150;
        float endY = screenHeight - animatedView.getHeight() - 100;
        float endSize = 2.0f;

        // define multiple animators, e.g. one for the x position, one for the y position, and one for the x scale factor
        ObjectAnimator animX = ObjectAnimator.ofFloat(animatedView, "x", endX);
        animX.setDuration(duration);
        ObjectAnimator animY = ObjectAnimator.ofFloat(animatedView, "y", endY);
        animY.setDuration(duration);
        ObjectAnimator animSize = ObjectAnimator.ofFloat(animatedView, "scaleX", endSize);
        animSize.setDuration(duration);

        // Option 1: start the animators separately
        // animX.start();
        // animY.start();
        // animSize.start();

        // Option 2: combine the animators in an AnimatorSet
        AnimatorSet animSet = new AnimatorSet();
        animSet.playTogether(animX,animY,animSize);
        // animSet.setDuration(duration)    // as an alternative to setting the durations separately
        animSet.start();

    }

    public void reset(View v) {

        setContentView(R.layout.multiple_properties_animation);

    }

}
