package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 15.11.2017

// This activity demonstrates the animation of a view with varying speeds and timing behaviors

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.animation.ObjectAnimator;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.AnticipateInterpolator;
import android.view.animation.AnticipateOvershootInterpolator;
import android.view.animation.BounceInterpolator;
import android.view.animation.CycleInterpolator;
import android.view.animation.DecelerateInterpolator;
import android.view.animation.OvershootInterpolator;
import android.widget.TextView;


public class NonConstantSpeedAnimationActivity extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.nonconstant_speed_animation);

    }

    public void startAnimation(View v) {

        int duration = 2000; // duration of the animation
        int screenHeight = getResources().getDisplayMetrics().heightPixels;
        float endY = 3*screenHeight/4;

        TextView animatedViews[] = new TextView[8];
        animatedViews[0] = (TextView) findViewById(R.id.animview0);
        animatedViews[1] = (TextView) findViewById(R.id.animview1);
        animatedViews[2] = (TextView) findViewById(R.id.animview2);
        animatedViews[3] = (TextView) findViewById(R.id.animview3);
        animatedViews[4] = (TextView) findViewById(R.id.animview4);
        animatedViews[5] = (TextView) findViewById(R.id.animview5);
        animatedViews[6] = (TextView) findViewById(R.id.animview6);
        animatedViews[7] = (TextView) findViewById(R.id.animview7);

        // generate ObjectAnimators to control the animations: movement into the y direction

        ObjectAnimator objectAnimators[] = new ObjectAnimator[animatedViews.length];
        for (int i=0;i<objectAnimators.length;i++) {
            objectAnimators[i] = ObjectAnimator.ofFloat(animatedViews[i], "y", endY);
            objectAnimators[i].setDuration(duration);
        }

        // set TimeInterpolators to control the timing of the animations:
        //   different TimeInterpolators lead to different timing behaviors
        //   (for a detailed explanation of interpolators, see SelfDefinedTimingAnimationActivity.java)

        objectAnimators[0].setInterpolator(new AccelerateInterpolator());
        objectAnimators[1].setInterpolator(new AccelerateDecelerateInterpolator());
        objectAnimators[2].setInterpolator(new DecelerateInterpolator());

        objectAnimators[3].setInterpolator(new AnticipateInterpolator());
        objectAnimators[4].setInterpolator(new AnticipateOvershootInterpolator());
        objectAnimators[5].setInterpolator(new OvershootInterpolator());
        objectAnimators[6].setInterpolator(new BounceInterpolator());

        objectAnimators[7].setInterpolator(new CycleInterpolator(3));

        // start the animations

        for (int i=0;i<objectAnimators.length;i++)
           objectAnimators[i].start();

    }

    public void reset(View v) {

        setContentView(R.layout.nonconstant_speed_animation);

    }

}
