package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 15.11.2017

// This activity demonstrates how to specify the timing of an animation

import android.animation.ObjectAnimator;
import android.animation.TimeInterpolator;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;


public class SelfDefinedTimingAnimationActivity extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.self_defined_timing_animation);

    }

    public void startAnimation(View v) {

        int duration = 2000;  // duration of the animation in ms
        int screenHeight = getResources().getDisplayMetrics().heightPixels;

        TextView animatedView = (TextView) findViewById(R.id.view5);
        float endY = screenHeight - animatedView.getHeight() - 300;

        ObjectAnimator animY = ObjectAnimator.ofFloat(animatedView, "y", endY);

        animY.setInterpolator(new MyTimeInterpolator()); // see detailed comment below
        animY.setDuration(duration);
        animY.start();

    }

    class MyTimeInterpolator implements TimeInterpolator {

    // The TimeInterpolator defines a getInterpolation() method, which receives the actual (= real) time
    // as a parameter. More precisely speaking: The method receives the fraction of the animation duration
    // passed so far, as a value between 0.0 and 1.0. getInterpolation() maps this real time value to a
    // virtual time value, also between 0.0 and 1.0. This virtual time value is then used by the ObjectAnimator
    // to calculate the current value of the object property.
    //  (The ObjectAnimator calls the function evaluate() of an evaluator. evaluate() calculates the value of the
    //   animated object property from the current time, i.e. fraction of the animation duration passed so far.
    //   The calculated value might e.g. be the current position of a View on its way from its initial
    //   position to its target position. The function is usually a linear one, e.g.
    //     current position = initial position + fraction passed * (target position - initial position). )
    // Mapping the real time to a virtual time before calling evaluate() allows to stretch or compress the time axis
    // (so to speak) under full program control, thus causing the ObjectAnimator to animate properties not linearly
    // but with varying speeds.
    //
    // This principle is best understood by having a look at the getInterpolation() method defined below.
    // It maps all real time values smaller than 0.25 to the virtual time 0.0. Thus, during the first quarter of the
    // animation, the ObjectAnimator takes this virtual time 0.0 as an input to its evaluate() function that calculates
    // the current object property value.
    // If, as it is the case here, the property is the position of a View on the screen and the
    // ObjectAnimator calculates the position by the linear function given above, then this position will not change
    // during the first quarter of the animation - it will always be the initial position of the view (i.e. its
    // position at time 0). In the second quarter, the view will be displayed at a position one quarter of its way to its
    // target position, because the interpolator then feeds the time value 0.25 into evaluate(), and so on.

        @Override
        public float getInterpolation(float input) {

            if (input < 0.25) return 0.0f;

            if (input < 0.5) return 0.25f;

            if (input < 0.75) return 0.5f;

            if (input < 0.99) return 0.75f;

            return 1.0f;

        }

    }

    public void reset(View v) {

        setContentView(R.layout.self_defined_timing_animation);

    }

}
