package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 15.11.2017

// This activity demonstrates how to use ObjectAnimators to animate various single properties of a view with constant speed.

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.animation.ObjectAnimator;
import android.widget.LinearLayout;
import android.widget.TextView;

public class SinglePropertyAnimationActivity extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.single_property_animation);

    }

    public void startAnimation(View v) {

        int duration = 1500;  // duration of the animation in ms
        int screenWidth = getResources().getDisplayMetrics().widthPixels;
        int screenHeight = getResources().getDisplayMetrics().heightPixels;

        // animate movement into x direction
        TextView animatedView1 = (TextView) findViewById(R.id.view1);
        float endX = screenWidth - animatedView1.getWidth() - 100;  // target x position of the object
        ObjectAnimator animX = ObjectAnimator.ofFloat(animatedView1, "x", endX);
               // ObjectAnimator to animate the x position of the object
               // ("x" = the animator shall call the "setX()" method of the animated object to modify its properties)
        animX.setDuration(duration);
        animX.start();

        // animate movement into y direction
        TextView animatedView2 = (TextView) findViewById(R.id.view2);
        float endY = screenHeight - animatedView2.getHeight() - 250;  // target y position of the object
        ObjectAnimator animY = ObjectAnimator.ofFloat(animatedView2, "y", endY);
               // ObjectAnimator to animate the y position of the object
               // ("y" = the animator shall call the "setY()" method of the animated object to modify its properties)
        animY.setDuration(duration);
        animY.start();

        // scale
        TextView animatedView3 = (TextView) findViewById(R.id.view3);
        float endSize = 2.0f;  // target size of the object (i.e. double)
        ObjectAnimator animSizeX = ObjectAnimator.ofFloat(animatedView3, "scaleX", endSize);
               // ObjectAnimator to animate the x scale factor of the object
               // ("scaleX" = the animator shall call the "setScaleX()" method of the animated object to modify its properties)
        animSizeX.setDuration(duration);
        animSizeX.setRepeatCount(3);  // repeat the scaling three times ...
        animSizeX.setRepeatMode(ObjectAnimator.REVERSE); // ... and also backwards, i.e. let the object grow and shrink
        animSizeX.start();

        // alternative: scale text size
        // TextView animatedView3 = (TextView) findViewById(R.id.view3);
        // float endSize = animatedView3.getTextSize()+20;
        // ObjectAnimator animSize = ObjectAnimator.ofFloat(animatedView3, "textSize", endSize);
        // animSize.setDuration(duration);
        // animSize.start();

        // rotate
        TextView animatedView4 = (TextView) findViewById(R.id.view4);
        ObjectAnimator animRot = ObjectAnimator.ofFloat(animatedView4, "rotation", 360);
               // ObjectAnimator to rotate the object
               // ("rotation" = the animator shall call the "setRotation()" method of the animated object to modify its properties)
        animRot.setDuration(duration);
        animRot.setRepeatCount(3);  // repeat the rotation three times
        animRot.start();

        // fade
        TextView animatedView5 = (TextView) findViewById(R.id.view5);
        ObjectAnimator animFade = ObjectAnimator.ofFloat(animatedView5, "alpha", 0);
               // ObjectAnimator to let the object vanish
               // ("alpha" = the animator shall call the "setAlpha()" method of the animated object to modify its properties)
        animFade.setDuration(duration);
        animFade.start();

        // animate color
        LinearLayout animatedView6 = (LinearLayout) findViewById(R.id.layout);
        int colorFrom = Color.WHITE;  // initial color
        int colorTo = Color.YELLOW;   // target color
        ObjectAnimator animCol = ObjectAnimator.ofInt(animatedView6, "backgroundColor", colorFrom, colorTo);
               // ObjectAnimator to let the object vanish
               // ("backgroundColor" = the animator shall call the "setBackgroundColor()" method of the animated object to modify its properties)
        animCol.setDuration(2*duration);
        animCol.setRepeatCount(1);  // repeat the animation once ...
        animCol.setRepeatMode(ObjectAnimator.REVERSE);  // ... backwards, i.e. the screen is white again at the end
        animCol.start();

    }

    public void reset(View v) {

        setContentView(R.layout.single_property_animation);

    }

}
